@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package YAML::PP;
 
-our $VERSION = '0.023_003'; # TRIAL VERSION
+our $VERSION = '0.026_001'; # TRIAL VERSION
 
 use YAML::PP::Schema;
 use YAML::PP::Schema::JSON;
@@ -26,9 +26,11 @@ sub new {
     my $schemas = delete $args{schema} || ['+'];
     my $cyclic_refs = delete $args{cyclic_refs} || 'allow';
     my $indent = delete $args{indent};
+    my $width = delete $args{width};
     my $writer = delete $args{writer};
     my $header = delete $args{header};
     my $footer = delete $args{footer};
+    my $duplicate_keys = delete $args{duplicate_keys};
     my $yaml_version = $class->_arg_yaml_version(delete $args{yaml_version});
     my $default_yaml_version = $yaml_version->[0];
     my $version_directive = delete $args{version_directive};
@@ -36,6 +38,7 @@ sub new {
     my $parser = delete $args{parser};
     my $emitter = delete $args{emitter} || {
         indent => $indent,
+        width => $width,
         writer => $writer,
     };
     if (keys %args) {
@@ -65,6 +68,7 @@ sub new {
         parser => $parser,
         default_yaml_version => $default_yaml_version,
         preserve => $preserve,
+        duplicate_keys => $duplicate_keys,
     );
     my $dumper = YAML::PP::Dumper->new(
         schema => $default_schema,
@@ -202,6 +206,7 @@ sub preserved_mapping {
     %$data = %$hash;
     my $t = tied %$data;
     $t->{style} = $args{style};
+    $t->{alias} = $args{alias};
     return $data;
 }
 
@@ -212,6 +217,7 @@ sub preserved_sequence {
     push @$data, @$array;
     my $t = tied @$data;
     $t->{style} = $args{style};
+    $t->{alias} = $args{alias};
     return $data;
 }
 
@@ -344,6 +350,7 @@ sub EXTEND {}
 package YAML::PP::Preserve::Scalar;
 
 use overload
+    fallback => 1,
     '+' => \&value,
     '""' => \&value,
     'bool' => \&value,
@@ -357,7 +364,8 @@ sub new {
 }
 sub value { $_[0]->{value} }
 sub tag { $_[0]->{tag} }
-sub style { $_[0]->{style} }
+sub style { $_[0]->{style} || 0 }
+sub alias { $_[0]->{alias} }
 
 1;
 
@@ -448,7 +456,7 @@ Some utility scripts, mostly useful for debugging:
 
 YAML::PP is a modular YAML processor.
 
-It aims to support C<YAML 1.2> and C<YAML 1.1>. See L<http://yaml.org/>.
+It aims to support C<YAML 1.2> and C<YAML 1.1>. See L<https://yaml.org/>.
 Some (rare) syntax elements are not yet supported and documented below.
 
 YAML is a serialization language. The YAML input is called "YAML Stream".
@@ -509,10 +517,19 @@ Options:
 
 Values: C<perl> (currently default), C<JSON::PP>, C<boolean>
 
+This option is for loading and dumping.
+
+Note that when dumping, only the chosen boolean style will be recognized.
+So if you choose C<JSON::PP>, C<boolean> objects will not be recognized
+as booleans and will be dumped as ordinary objects (if you enable the
+Perl schema).
+
 =item schema
 
 Default: C<['Core']>
 
+This option is for loading and dumping.
+
 Array reference. Here you can define what schema to use.
 Supported standard Schemas are: C<Failsafe>, C<JSON>, C<Core>, C<YAML1_1>.
 
@@ -525,6 +542,8 @@ Additionally you can add further schemas, for example C<Merge>.
 
 Default: 'allow' but will be switched to fatal in the future for safety!
 
+This option is for loading only.
+
 Defines what to do when a cyclic reference is detected when loading.
 
     # fatal  - die
@@ -532,28 +551,76 @@ Defines what to do when a cyclic reference is detected when loading.
     # ignore - replace with undef
     # allow  - Default
 
+=item duplicate_keys
+
+Default: 1
+
+Since version 0.026
+
+This option is for loading.
+
+NOTE: THIS OPTION WILL BE SET TO 0 IN THE NEXT RELEASE.
+
+The YAML Spec says duplicate mapping keys should be forbidden.
+
+When set to true, duplicate keys in mappings are allowed (and will overwrite
+the previous key).
+
+When set to false, duplicate keys will result in an error when loading.
+
+This is especially useful when you have a longer mapping and don't see
+the duplicate key in your editor:
+
+    ---
+    a: 1
+    b: 2
+    # .............
+    a: 23 # error
+
 =item indent
 
 Default: 2
 
+This option is for dumping.
+
 Use that many spaces for indenting
 
+=item width
+
+Since version 0.025
+
+Default: 80
+
+This option is for dumping.
+
+Maximum columns when dumping.
+
+This is only respected when dumping flow collections right now.
+
+in the future it will be used also for wrapping long strings.
+
 =item header
 
 Default: 1
 
+This option is for dumping.
+
 Print document heaader C<--->
 
 =item footer
 
 Default: 0
 
+This option is for dumping.
+
 Print document footer C<...>
 
 =item yaml_version
 
 Since version 0.020
 
+This option is for loading and dumping.
+
 Default: C<1.2>
 
 Note that in this case, a directive C<%YAML 1.1> will basically be ignored
@@ -600,6 +667,8 @@ schema.
 
 Since version 0.020
 
+This option is for dumping.
+
 Default: 0
 
 Print Version Directive C<%YAML 1.2> (or C<%YAML 1.1>) on top of each YAML
@@ -611,6 +680,8 @@ Since version 0.021
 
 Default: false
 
+This option is for loading and dumping.
+
 Preserving scalar styles is still experimental.
 
     use YAML::PP::Common qw/ PRESERVE_ORDER PRESERVE_SCALAR_STYLE /;
@@ -621,9 +692,12 @@ Preserving scalar styles is still experimental.
     # Preserve the quoting style of scalars
     my $yp = YAML::PP->new( preserve => PRESERVE_SCALAR_STYLE );
 
-    # Preserve block/flow style
+    # Preserve block/flow style (since 0.024)
     my $yp = YAML::PP->new( preserve => PRESERVE_FLOW_STYLE );
 
+    # Preserve alias names (since 0.027)
+    my $yp = YAML::PP->new( preserve => PRESERVE_ALIAS );
+
     # Combine, e.g. preserve order and scalar style
     my $yp = YAML::PP->new( preserve => PRESERVE_ORDER | PRESERVE_SCALAR_STYLE );
 
@@ -641,14 +715,17 @@ If you load the following input:
     - |
       literal
     ---
-    block mapping:
+    block mapping: &alias
       flow sequence: [a, b]
+    same mapping: *alias
     flow mapping: {a: b}
 
+
 with this code:
 
     my $yp = YAML::PP->new(
-        preserve => PRESERVE_ORDER | PRESERVE_SCALAR_STYLE | PRESERVE_FLOW_STYLE
+        preserve => PRESERVE_ORDER | PRESERVE_SCALAR_STYLE
+                    | PRESERVE_FLOW_STYLE | PRESERVE_ALIAS
     );
     my ($hash, $styles, $flow) = $yp->load_file($file);
     $yp->dump_file($hash, $styles, $flow);
@@ -656,6 +733,21 @@ with this code:
 Then dumping it will return the same output.
 Only folded block scalars '>' cannot preserve the style yet.
 
+Note that YAML allows repeated definition of anchors. They cannot bw preserved
+with YAML::PP right now. Example:
+
+    ---
+    - &seq [a]
+    - *seq
+    - &seq [b]
+    - *seq
+
+Because the data could be shuffled before dumping again, the anchor definition
+could be broken. In this case repeated anchor names will be discarded when
+loading and dumped with numeric anchors like usual.
+
+Implementation:
+
 When loading, hashes will be tied to an internal class
 (C<YAML::PP::Preserve::Hash>) that keeps the key order.
 
@@ -729,6 +821,8 @@ L<YAML::PP::Writer> and output a string.
 
 =head2 preserved_scalar
 
+Since version 0.024
+
 Experimental. Please report bugs or let me know this is useful and works.
 
 You can define a certain scalar style when dumping data.
@@ -753,6 +847,8 @@ it right for all cases. It's also a matter of taste.
 
 =head2 preserved_mapping, preserved_sequence
 
+Since version 0.024
+
 Experimental. Please report bugs or let me know this is useful and works.
 
 With this you can define which nodes are dumped with the more compact flow
@@ -1210,7 +1306,7 @@ and created a matrix view.
 
 L<https://github.com/perlpunk/yaml-test-matrix>
 
-You can find the latest build at L<http://matrix.yaml.io>
+You can find the latest build at L<https://matrix.yaml.io>
 
 As of this writing, the test matrix only contains valid test cases.
 Invalid ones will be added.
@@ -1247,6 +1343,8 @@ Felix answered countless questions about the YAML Specification.
 
 =item L<YAML::LibYAML::API>
 
+=item L<https://www.yaml.info>
+
 =back
 
 =head1 SPONSORS
